"""
structures.py

Fancy structure functions, classes, etc.

:author:        Stephen Stauts
:created:       07/03/2020   (DD/MM/YYYY)
:copyright:     © 2020 Stephen Stauts. All Rights Reserved.
"""

# Standard Packages
import os
from pathlib import Path

# Dependency Packages

# Local Packages and Modules


class NestedDict(dict):
    """
    Dict class able to return a value nested among dictionaries, pointed to by a key tuple.
    For example:
        d = { 'one': { 'two': 2, 'three': 3 } }
        nd = NestedDict(d)
        nd['one'] ==> { 'two': 2, 'three': 3 }
        nd['one', 'two'] ==> 2

    Credit must be given to finding this class on https://stackoverflow.com/a/30648524.
    You da real MVP.
    """

    def __getitem__(self, t_keys):
        """
        Get a value based on a tuple of keys, or a normal dict access for a single key.

        :param tuple t_keys:    Tuple of keys leading to the desired value.
        :return str:            The nested value or None if it does not exist.
        """
        if not isinstance(t_keys, tuple):
            return super(NestedDict, self).__getitem__(t_keys)

        value = self
        for key in t_keys:
            value = value.get(key)
        return value
